"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Hostable = void 0;
const protocol_1 = require("@skeldjs/protocol");
const util_1 = require("@skeldjs/util");
const constant_1 = require("@skeldjs/constant");
const events_1 = require("@skeldjs/events");
const objects_1 = require("./objects");
const Networkable_1 = require("./Networkable");
const PlayerData_1 = require("./PlayerData");
const events_2 = require("./events");
const endgame_1 = require("./endgame");
const roles_1 = require("./roles");
/**
 * Represents an object capable of hosting games.
 *
 * See {@link HostableEvents} for events to listen to.
 */
class Hostable extends events_1.EventEmitter {
    constructor(config = {}) {
        super();
        this.config = config;
        this.last_fixed_update = Date.now();
        this._incrNetId = 0;
        this._destroyed = false;
        this.code = 0;
        this.gameState = constant_1.GameState.NotStarted;
        this.hostId = -1;
        this.counter = -1;
        this.privacy = "private";
        this.settings = new protocol_1.GameSettings;
        this.objectList = [];
        this.players = new Map;
        this.netobjects = new Map;
        this.messageStream = [];
        this.shipStatus = undefined;
        this.meetingHud = undefined;
        this.lobbyBehaviour = undefined;
        this.gameData = undefined;
        this.voteBanSystem = undefined;
        this.registeredPrefabs = new Map([
            [constant_1.SpawnType.SkeldShipStatus, [objects_1.SkeldShipStatus]],
            [constant_1.SpawnType.MeetingHud, [objects_1.MeetingHud]],
            [constant_1.SpawnType.LobbyBehaviour, [objects_1.LobbyBehaviour]],
            [constant_1.SpawnType.GameData, [objects_1.GameData, objects_1.VoteBanSystem]],
            [constant_1.SpawnType.Player, [objects_1.PlayerControl, objects_1.PlayerPhysics, objects_1.CustomNetworkTransform]],
            [constant_1.SpawnType.MiraShipStatus, [objects_1.MiraShipStatus]],
            [constant_1.SpawnType.Polus, [objects_1.PolusShipStatus]],
            [constant_1.SpawnType.AprilShipStatus, [objects_1.AprilShipStatus]],
            [constant_1.SpawnType.Airship, [objects_1.AirshipStatus]]
        ]);
        this.registeredRoles = new Map([
            [constant_1.RoleType.Crewmate, roles_1.CrewmateRole],
            [constant_1.RoleType.Engineer, roles_1.EngineerRole],
            [constant_1.RoleType.GuardianAngel, roles_1.GuardianAngelRole],
            [constant_1.RoleType.Impostor, roles_1.ImpostorRole],
            [constant_1.RoleType.Scientist, roles_1.ScientistRole],
            [constant_1.RoleType.Shapeshifter, roles_1.ShapeshifterRole]
        ]);
        this.endGameIntents = [];
        if (config.doFixedUpdate) {
            this._interval = setInterval(() => this.FixedUpdate(), Hostable.FixedUpdateInterval);
        }
    }
    destroy() {
        if (this._interval)
            clearInterval(this._interval);
        this._interval = undefined;
        this._destroyed = true;
        this.gameState = constant_1.GameState.Destroyed;
    }
    getNextNetId() {
        this._incrNetId++;
        return this._incrNetId;
    }
    /**
     * The current client in the room.
     */
    get myPlayer() {
        return undefined;
    }
    /**
     * The host of the room.
     */
    get host() {
        return this.players.get(this.hostId);
    }
    get destroyed() {
        return this._destroyed;
    }
    /**
     * Whether or not the current client is the host of the room.
     */
    get hostIsMe() {
        return false;
    }
    async broadcast(gamedata, payloads = [], include, exclude, reliable = true) { }
    async FixedUpdate() {
        var _a;
        const delta = Date.now() - this.last_fixed_update;
        this.last_fixed_update = Date.now();
        for (const [, component] of this.netobjects) {
            if (component &&
                (component.ownerId === ((_a = this.myPlayer) === null || _a === void 0 ? void 0 : _a.clientId) || this.hostIsMe)) {
                component.FixedUpdate(delta / 1000);
                if (component.dirtyBit) {
                    component.PreSerialize();
                    const writer = util_1.HazelWriter.alloc(0);
                    if (component.Serialize(writer, false)) {
                        this.messageStream.push(new protocol_1.DataMessage(component.netId, writer.buffer));
                    }
                    component.dirtyBit = 0;
                }
            }
        }
        if (this.endGameIntents.length) {
            const endGameIntents = this.endGameIntents;
            this.endGameIntents = [];
            if (this.hostIsMe) {
                for (let i = 0; i < endGameIntents.length; i++) {
                    const intent = endGameIntents[i];
                    const ev = await this.emit(new events_2.RoomEndGameIntentEvent(this, intent.name, intent.reason, intent.metadata));
                    if (ev.canceled) {
                        endGameIntents.splice(i, 1);
                        i--;
                    }
                }
                if (endGameIntents[0]) {
                    this.endGame(endGameIntents[0].reason);
                }
            }
        }
        const ev = await this.emit(new events_2.RoomFixedUpdateEvent(this, this.messageStream, delta / 1000));
        if (this.messageStream.length) {
            const stream = this.messageStream;
            this.messageStream = [];
            if (!ev.canceled)
                await this.broadcast(stream);
        }
    }
    /**
     * Resolve a player by some identifier.
     * @param player The identifier to resolve to a player.
     * @returns The resolved player.
     * @example
     *```typescript
     * // Resolve a player by their client id.
     * const player = room.resolvePlayer(11013);
     * ```
     */
    resolvePlayer(player) {
        if (player instanceof PlayerData_1.PlayerData)
            return player;
        const clientId = this.resolvePlayerClientID(player);
        if (clientId === undefined)
            return undefined;
        return this.players.get(clientId);
    }
    /**
     * Resolve a player ID by some identifier.
     * @param player The identifier to resolve to a player ID.
     * @returns The resolved player ID.
     */
    resolvePlayerId(player) {
        if (typeof player === "undefined") {
            return undefined;
        }
        if (typeof player === "number") {
            return player;
        }
        return player.playerId;
    }
    /**
     * Resolve a client id by some identifier.
     * @param player The identifier to resolve to a client ID.
     * @returns The resolved client ID.
     */
    resolvePlayerClientID(player) {
        if (typeof player === "undefined") {
            return undefined;
        }
        if (typeof player === "number") {
            return player;
        }
        if (player instanceof Networkable_1.Networkable) {
            return player.ownerId;
        }
        if (player instanceof PlayerData_1.PlayerData) {
            return player.clientId;
        }
        return undefined;
    }
    /**
     * Set the code of the room.
     * @example
     *```typescript
     * room.setCode("ABCDEF");
     * ```
     */
    setCode(code) {
        if (typeof code === "string") {
            return this.setCode(util_1.GameCode.convertStringToInt(code));
        }
        this.code = code;
    }
    _setPrivacy(privacy) {
        this.privacy = privacy;
    }
    /**
     * Change the the privacy of the room.
     * @param tag The tag to change.
     * @param value The new value of the tag.
     * @example
     *```typescript
     * room.setAlterGameTag(AlterGameTag.ChangePrivacy, 1); // 0 for private, 1 for public.
     * ```
     */
    async setPrivacy(privacy) {
        const oldPrivacy = this.privacy;
        this._setPrivacy(privacy);
        const ev = await this.emit(new events_2.RoomSetPrivacyEvent(this, undefined, oldPrivacy, privacy));
        this._setPrivacy(ev.alteredPrivacy);
        if (ev.alteredPrivacy !== oldPrivacy) {
            await this.broadcast([], [
                new protocol_1.AlterGameMessage(this.code, constant_1.AlterGameTag.ChangePrivacy, this.privacy === "public" ? 1 : 0),
            ]);
        }
    }
    /**
     * Change the settings of the room. If the host, it will broadcast these changes.
     * @param settings The settings to set to (Can be partial).
     * @example
     *```typescript
     * room.syncSettings({
     *   crewmateVision: 0.5,
     *   votingTime: 120
     * });
     * ```
     */
    setSettings(settings) {
        this.settings.patch(settings);
        if (this.hostIsMe && this.host && this.host.control) {
            this.host.control.syncSettings(this.settings);
        }
    }
    spawnNecessaryObjects() {
        var _a;
        if (!this.lobbyBehaviour && this.gameState === constant_1.GameState.NotStarted) {
            this.spawnPrefabOfType(constant_1.SpawnType.LobbyBehaviour, -2);
        }
        if (!this.shipStatus && this.gameState === constant_1.GameState.Started) {
            const shipPrefabs = [
                constant_1.SpawnType.SkeldShipStatus,
                constant_1.SpawnType.MiraShipStatus,
                constant_1.SpawnType.Polus,
                constant_1.SpawnType.AprilShipStatus,
                constant_1.SpawnType.Airship
            ];
            this.spawnPrefabOfType(shipPrefabs[(_a = this.settings) === null || _a === void 0 ? void 0 : _a.map] || 0, -2);
        }
        if (!this.gameData) {
            this.spawnPrefabOfType(constant_1.SpawnType.GameData, -2);
        }
    }
    /**
     * Set the host of the room. If the current client is the host, it will conduct required host changes.
     * e.g. Spawning objects if they are not already spawned.
     * @param host The new host of the room.
     */
    async setHost(host) {
        const before = this.hostId;
        const resolvedId = this.resolvePlayerClientID(host);
        if (!resolvedId)
            return;
        this.hostId = resolvedId;
        if (this.hostIsMe) {
            this.spawnNecessaryObjects();
        }
        if (before !== this.hostId && this.host) {
            await this.host.emit(new events_2.PlayerSetHostEvent(this, this.host));
        }
    }
    /**
     * Handle when a client joins the game.
     * @param clientId The ID of the client that joined the game.
     */
    async handleJoin(joinInfo) {
        const cachedPlayer = this.players.get(joinInfo.clientId);
        if (cachedPlayer)
            return cachedPlayer;
        const player = new PlayerData_1.PlayerData(this, joinInfo.clientId, joinInfo.playerName, joinInfo.platform, joinInfo.playerLevel, joinInfo.friendCode, joinInfo.puid);
        this.players.set(joinInfo.clientId, player);
        if (this.hostIsMe) {
            this.spawnNecessaryObjects();
        }
        await player.emit(new events_2.PlayerJoinEvent(this, player));
        return player;
    }
    checkPlayersRemaining() {
        if (!this.gameData || !this.shipStatus)
            return;
        let aliveImpostors = 0;
        let totalImpostors = 0;
        let totalCrewmates = 0;
        for (const [, playerInfo] of this.gameData.players) {
            if (!playerInfo.isDisconnected) {
                if (playerInfo.isImpostor) {
                    totalImpostors++;
                }
                else {
                    totalCrewmates++;
                }
                if (!playerInfo.isDead) {
                    if (playerInfo.isImpostor) {
                        aliveImpostors++;
                    }
                }
            }
        }
        const reason = totalImpostors === 0
            ? constant_1.GameOverReason.ImpostorDisconnect
            : totalCrewmates <= aliveImpostors
                ? constant_1.GameOverReason.HumansDisconnect
                : -1;
        if (reason !== -1) {
            this.registerEndGameIntent(new endgame_1.EndGameIntent(endgame_1.AmongUsEndGames.PlayersDisconnect, reason, {
                aliveImpostors,
                totalImpostors,
                totalCrewmates
            }));
        }
    }
    /**
     * Handle when a client leaves the game.
     * @param resolvable The client that left the game.
     */
    async handleLeave(resolvable) {
        var _a, _b;
        const player = this.resolvePlayer(resolvable);
        if (!player)
            return null;
        if (player.playerId !== undefined) {
            if (this.gameState === constant_1.GameState.Started) {
                const gamedataEntry = (_a = this.gameData) === null || _a === void 0 ? void 0 : _a.players.get(player.playerId);
                if (gamedataEntry) {
                    gamedataEntry.setDisconnected(true);
                }
                this.checkPlayersRemaining();
            }
            else {
                (_b = this.gameData) === null || _b === void 0 ? void 0 : _b.players.delete(player.playerId);
            }
            if (this.hostIsMe && this.meetingHud) {
                for (const [, voteState] of this.meetingHud.voteStates) {
                    const voteStatePlayer = voteState.player;
                    if (voteStatePlayer && voteState.votedForId === player.playerId) {
                        this.meetingHud.clearVote(voteStatePlayer);
                    }
                }
            }
        }
        if (this.voteBanSystem && this.voteBanSystem.voted.get(player.clientId)) {
            this.voteBanSystem.voted.delete(player.clientId);
        }
        if (player.control) {
            for (const component of player.control.components) {
                this.despawnComponent(component);
            }
        }
        this.players.delete(player.clientId);
        this.emitSync(new events_2.PlayerLeaveEvent(this, player));
        return player;
    }
    /**
     * Handle when the game is started.
     */
    async handleStart() {
        var _a, _b, _c, _d, _e;
        if (this.gameState === constant_1.GameState.Started)
            return;
        this.gameState = constant_1.GameState.Started;
        if (this.hostIsMe) {
            await Promise.all([
                Promise.race([
                    Promise.all([...this.players.values()].map((player) => {
                        if (player.isReady) {
                            return Promise.resolve();
                        }
                        return new Promise((resolve) => {
                            player.once("player.ready", () => {
                                resolve();
                            });
                        });
                    })),
                    util_1.sleep(3000),
                ]),
                (_a = this.myPlayer) === null || _a === void 0 ? void 0 : _a.setReady(),
            ]);
            const removes = [];
            for (const [clientId, player] of this.players) {
                if (!player.isReady) {
                    await this.handleLeave(player);
                    removes.push(clientId);
                }
            }
            if (removes.length) {
                await this.broadcast([], removes.map(clientId => {
                    return new protocol_1.RemovePlayerMessage(this.code, clientId, constant_1.DisconnectReason.Error);
                }));
            }
            if (this.lobbyBehaviour)
                this.lobbyBehaviour.despawn();
            const shipPrefabs = [
                constant_1.SpawnType.SkeldShipStatus,
                constant_1.SpawnType.MiraShipStatus,
                constant_1.SpawnType.Polus,
                constant_1.SpawnType.AprilShipStatus,
                constant_1.SpawnType.Airship
            ];
            await this.emit(new events_2.RoomGameStartEvent(this));
            this.spawnPrefabOfType(shipPrefabs[(_b = this.settings) === null || _b === void 0 ? void 0 : _b.map] || 0, -2);
            await ((_c = this.shipStatus) === null || _c === void 0 ? void 0 : _c.assignRoles());
            await ((_d = this.shipStatus) === null || _d === void 0 ? void 0 : _d.assignTasks());
            if (this.shipStatus) {
                for (const [, player] of this.players) {
                    this.shipStatus.spawnPlayer(player, true);
                }
            }
        }
        else {
            await this.emit(new events_2.RoomGameStartEvent(this));
            await ((_e = this.myPlayer) === null || _e === void 0 ? void 0 : _e.setReady());
        }
    }
    async _handleEnd(reason) {
        this.gameState = constant_1.GameState.Ended;
        this.players.clear();
        for (const [objId] of this.players) {
            this.players.delete(objId);
        }
        if (this.hostIsMe) {
            for (const [, component] of this.netobjects) {
                component.despawn();
            }
        }
        await this.emit(new events_2.RoomGameEndEvent(this, reason));
    }
    /**
     * Handle when the game is ended.
     * @param reason The reason for why the game ended.
     */
    async handleEnd(reason) {
        await this._handleEnd(reason);
    }
    async endGame(reason) {
        if (this.gameState !== constant_1.GameState.Started)
            return;
        this.gameState = constant_1.GameState.Ended;
        await this.broadcast([], [
            new protocol_1.EndGameMessage(this.code, reason, false)
        ]);
    }
    /**
     * Handle a client readying up.
     * @param player The client that readied.
     */
    async handleReady(player) {
        const resolved = this.resolvePlayer(player);
        if (resolved) {
            await resolved.setReady();
        }
    }
    /**
     * Spawn a component (Not broadcasted to all clients, see {@link Hostable.spawnPrefabOfType}).
     * @param component The component being spawned.
     * @example
     * ```typescript
     * const meetinghud = new MeetingHud(
     *   this,
     *   this.getNextNetId(),
     *   ownerId,
     *   {
     *     dirtyBit: 0,
     *     states: new Map(),
     *   }
     * );
     *
     * this.spawnComponent(meetinghud);
     * ```
     */
    spawnComponent(component) {
        if (this.netobjects.get(component.netId)) {
            return;
        }
        if (component instanceof objects_1.LobbyBehaviour) {
            if (!this.lobbyBehaviour) {
                this.lobbyBehaviour = component; // (??)
            }
        }
        if (component instanceof objects_1.GameData) {
            if (!this.gameData) {
                this.gameData = component;
            }
        }
        if (component instanceof objects_1.InnerShipStatus) {
            if (!this.shipStatus) {
                this.shipStatus = component;
            }
        }
        if (component instanceof objects_1.VoteBanSystem) {
            this.voteBanSystem = component;
        }
        if (component instanceof objects_1.MeetingHud) {
            this.meetingHud = component;
        }
        this.netobjects.set(component.netId, component);
        component.emitSync(new events_2.ComponentSpawnEvent(this, component));
    }
    _despawnComponent(component) {
        this.netobjects.delete(component.netId);
        if (component.owner instanceof PlayerData_1.PlayerData) {
            if (component.owner.control === component) {
                component.owner.control = undefined;
            }
        }
        if (component === this.lobbyBehaviour) {
            this.lobbyBehaviour = undefined;
        }
        if (component === this.gameData) {
            this.gameData = undefined;
        }
        if (component === this.shipStatus) {
            this.shipStatus = undefined;
        }
        if (component === this.voteBanSystem) {
            this.voteBanSystem = undefined;
        }
        if (component === this.meetingHud) {
            this.meetingHud = undefined;
        }
        component.Destroy();
        const objectIdx = this.objectList.indexOf(component);
        if (objectIdx > -1) {
            this.objectList.splice(objectIdx, 1);
        }
        component.components.splice(component.components.indexOf(component), 1);
        component.emitSync(new events_2.ComponentDespawnEvent(this, component));
    }
    /**
     * Despawn a component.
     * @param component The component being despawned.
     * @example
     *```typescript
     * room.despawnComponent(room.meetinghud);
     * ```
     */
    despawnComponent(component) {
        this._despawnComponent(component);
        this.messageStream.push(new protocol_1.DespawnMessage(component.netId));
    }
    /**
     * Get an available player ID.
     * @returns The player ID that was found.
     * @example
     *```typescript
     * // Get an available player ID and add it to the gamedata.
     * const playerId = room.getAvailablePlayerID();
     * room.gamedata.add(playerId);
     * ```
     */
    getAvailablePlayerID() {
        if (!this.gameData)
            return 0;
        for (let i = 0;; i++) {
            if (!this.gameData.players.get(i)) {
                return i;
            }
        }
    }
    getExistingObjectSpawn() {
        const messages = [];
        for (const object of this.objectList) {
            messages.push(new protocol_1.SpawnMessage(object.spawnType, object.ownerId, object.flags, object.components.map(component => {
                const writer = util_1.HazelWriter.alloc(512);
                writer.write(component, true);
                writer.realloc(writer.cursor);
                return new protocol_1.ComponentSpawnData(component.netId, writer.buffer);
            })));
        }
        return messages;
    }
    spawnPrefab(spawnType, spawnPrefab, ownerId, flags, componentData = [], doBroadcast = true, doAwake = true) {
        var _a;
        const _ownerId = ownerId === undefined
            ? -2
            : typeof ownerId === "number" ? ownerId : ownerId.clientId;
        const ownerClient = this.players.get(_ownerId);
        const _flags = flags !== null && flags !== void 0 ? flags : (spawnType === constant_1.SpawnType.Player ? constant_1.SpawnFlag.IsClientCharacter : 0);
        let object;
        for (let i = 0; i < spawnPrefab.length; i++) {
            const component = new spawnPrefab[i](this, spawnType, ((_a = componentData === null || componentData === void 0 ? void 0 : componentData[i]) === null || _a === void 0 ? void 0 : _a.netId) || this.getNextNetId(), _ownerId, _flags, (componentData === null || componentData === void 0 ? void 0 : componentData[i]) instanceof protocol_1.ComponentSpawnData
                ? util_1.HazelReader.from(componentData[i].data)
                : componentData === null || componentData === void 0 ? void 0 : componentData[i], object);
            this._incrNetId = component.netId;
            if (this.netobjects.get(component.netId))
                continue;
            if (!object) {
                object = component;
                this.objectList.push(object);
            }
            this.spawnComponent(component);
            object.components.push(component);
        }
        if (!object)
            return;
        for (const component of object.components) {
            if (doAwake) {
                component.Awake();
            }
        }
        if (spawnType === constant_1.SpawnType.Player && ownerClient) {
            ownerClient.emitSync(new events_2.PlayerSpawnEvent(this, ownerClient));
        }
        if (this.hostIsMe && doBroadcast) {
            this.messageStream.push(new protocol_1.SpawnMessage(spawnType, object.ownerId, _flags, object.components.map((component) => {
                const writer = util_1.HazelWriter.alloc(512);
                writer.write(component, true);
                writer.realloc(writer.cursor);
                return new protocol_1.ComponentSpawnData(component.netId, writer.buffer);
            })));
        }
        if ((_flags & constant_1.SpawnFlag.IsClientCharacter) > 0 && ownerClient) {
            if (!ownerClient.control) {
                ownerClient.control = object;
                ownerClient.inScene = true;
            }
        }
        return object;
    }
    /**
     * Spawn a prefab of an object.
     * @param spawnType The type of object to spawn.
     * @param owner The owner or ID of the owner of the object to spawn.
     * @returns The object that was spawned.
     * @example
     *```typescript
     * room.spawnPrefab(SpawnType.Player, client.myPlayer);
     * ```
     */
    spawnPrefabOfType(spawnType, ownerId, flags, componentData, doBroadcast = true, doAwake = true) {
        const spawnPrefab = this.registeredPrefabs.get(spawnType);
        if (!spawnPrefab)
            throw new Error("Cannot spawn object of type: " + spawnType + " (not registered)");
        return this.spawnPrefab(spawnType, spawnPrefab, ownerId, flags, componentData, doBroadcast, doAwake);
    }
    /**
     * Create a fake player in the room that doesn't need a client to be connected
     * to own it.
     *
     * To dispose of the player, use `player.despawn()`.
     * @param isNew Whether or not the player should be seen jumping off of the seat in the lobby.
     * @returns The fake player created.
     */
    createFakePlayer(isNew = true) {
        const player = new PlayerData_1.PlayerData(this, 0, "dummy");
        const playerControl = this.spawnPrefabOfType(constant_1.SpawnType.Player, -2, undefined, !isNew ? [{ isNew: false }] : undefined);
        playerControl.player = player;
        player.control = playerControl;
        return player;
    }
    /**
     * Register a custom INO spawn object by its spawn type. Can also override
     * built-in objects.
     * @param spawnType The spawn type of the component as an integer.
     * @param components The components in the object. The first component should
     * be the main object which will inherit the rest of the components.
     */
    registerPrefab(spawnType, components) {
        if (components.length < 1) {
            throw new Error("Expected at least 1 component to create an INO prefab from.");
        }
        this.registeredPrefabs.set(spawnType, components);
    }
    /**
     * Register a role that can be assigned to a player.
     * @param roleCtr The role to register to the room.
     */
    registerRole(roleCtr) {
        this.registeredRoles.set(roleCtr.roleMetadata.roleType, roleCtr);
    }
    /**
     * Get a player by their player ID.
     * @param playerId The player ID of the player.
     * @returns The player that was found, or null if they do not exist.
     * @example
     * ```typescript
     * const player = room.getPlayerByPlayerId(1);
     * ```
     */
    getPlayerByPlayerId(playerId) {
        var _a;
        return (_a = this.getPlayerControlByPlayerId(playerId)) === null || _a === void 0 ? void 0 : _a.player;
    }
    getPlayerControlByPlayerId(playerId) {
        for (let i = 0; i < this.objectList.length; i++) {
            const object = this.objectList[i];
            if (object instanceof objects_1.PlayerControl) {
                if (object.playerId === playerId) {
                    return object;
                }
            }
        }
        return undefined;
    }
    /**
     * Get a player by one of their components' netIds.
     * @param netId The net ID of the component of the player to search.
     * @returns The player that was found, or null if they do not exist.
     * @example
     * ```typescript
     * const player = room.getPlayerByNetId(34);
     * ```
     */
    getPlayerByNetId(netId) {
        for (const [, player] of this.players) {
            if (!player.control)
                continue;
            if (player.control.components.find(component => component.netId === netId)) {
                return player;
            }
        }
        return undefined;
    }
    findPlayersWithName(displayName) {
        var _a;
        const players = [];
        for (const [, player] of this.players) {
            if (((_a = player.playerInfo) === null || _a === void 0 ? void 0 : _a.defaultOutfit.name) === displayName) {
                players.push(player);
            }
        }
        return players;
    }
    /**
     * Register an intent to end the game.
     * @param endGameIntent The intention to end the game.
     */
    registerEndGameIntent(endGameIntent) {
        this.endGameIntents.push(endGameIntent);
    }
    murderIsValid(murderer, victim) {
        var _a, _b, _c, _d;
        if (this.gameState !== constant_1.GameState.Started)
            return false;
        if (((_a = murderer.playerInfo) === null || _a === void 0 ? void 0 : _a.isDead) || ((_b = murderer.playerInfo) === null || _b === void 0 ? void 0 : _b.roleType.roleMetadata.roleTeam) !== constant_1.RoleTeamType.Impostor || ((_c = murderer.playerInfo) === null || _c === void 0 ? void 0 : _c.isDisconnected)) {
            return false;
        }
        if (!victim.playerInfo || victim.playerInfo.isDead || ((_d = victim.physics) === null || _d === void 0 ? void 0 : _d.isInVent)) {
            return false;
        }
        return true;
    }
}
exports.Hostable = Hostable;
/**
 * How often a FixedUpdate should be called.
 */
Hostable.FixedUpdateInterval = 1 / 50;
//# sourceMappingURL=Hostable.js.map