"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.InnerShipStatus = void 0;
const util_1 = require("@skeldjs/util");
const constant_1 = require("@skeldjs/constant");
const Networkable_1 = require("../Networkable");
const roles_1 = require("../roles");
const events_1 = require("../events");
function shuffleArray(array) {
    for (let i = array.length - 1; i > 0; i--) {
        const j = Math.floor(Math.random() * (i + 1));
        const temp = array[i];
        array[i] = array[j];
        array[j] = temp;
    }
}
class InnerShipStatus extends Networkable_1.Networkable {
    constructor(room, spawnType, netId, ownerid, flags, data) {
        super(room, spawnType, netId, ownerid, flags, data);
        if (!this.systems) {
            this.systems = new Map;
            this.Setup();
        }
        this.spawnRadius = 1.55;
        this.initialSpawnCenter = util_1.Vector2.null;
        this.meetingSpawnCenter = util_1.Vector2.null;
    }
    get owner() {
        return super.owner;
    }
    // eslint-disable-next-line @typescript-eslint/no-empty-function
    Setup() { }
    Deserialize(reader, spawn = false) {
        if (!this.systems) {
            this.systems = new Map;
            this.Setup();
        }
        while (reader.left) {
            const [tag, mreader] = reader.message();
            const system = this.systems.get(tag);
            if (system) {
                system.Deserialize(mreader, spawn);
            }
        }
    }
    /* eslint-disable-next-line */
    Serialize(writer, spawn = false) {
        for (const [, system] of this.systems) {
            if (system.dirty) {
                writer.begin(system.systemType);
                system.Serialize(writer, spawn);
                writer.end();
                system.dirty = false;
            }
        }
        this.dirtyBit = 0;
        return true;
    }
    async HandleRpc(rpc) {
        switch (rpc.messageTag) {
            case constant_1.RpcMessageTag.CloseDoorsOfType:
                await this._handleCloseDoorsOfType(rpc);
                break;
            case constant_1.RpcMessageTag.RepairSystem:
                await this._handleRepairSystem(rpc);
                break;
        }
    }
    async _handleCloseDoorsOfType(rpc) {
        const doors = this.systems.get(constant_1.SystemType.Doors);
        const doorsInRoom = this.getDoorsInRoom(rpc.systemId);
        for (const doorId of doorsInRoom) {
            doors.closeDoor(doorId);
        }
    }
    FixedUpdate(delta) {
        for (const [, system] of this.systems) {
            system.Detoriorate(delta);
        }
    }
    async _handleRepairSystem(rpc) {
        const system = this.systems.get(rpc.systemId);
        const player = this.room.getPlayerByNetId(rpc.netId);
        if (system && player) {
            await system.HandleRepair(player, rpc.amount, rpc);
        }
    }
    /**
     * Get all roles registered on the room that match a given filter.
     * @param filter The filter to match against.
     * @returns A list of all roles that satisfy the given filter.
     */
    matchRoles(settings, filter) {
        const filteredRoles = [];
        for (const [, roleCtr] of this.room.registeredRoles) {
            const roleChance = settings[roleCtr.roleMetadata.roleType];
            if (roleChance && filter(roleCtr, roleChance)) {
                filteredRoles.push(roleCtr);
            }
        }
        return filteredRoles;
    }
    /**
     * Specifically assign a pool of players to each role in a specific team.
     * @param playerPool The entire list of players that can be assigned a role,
     * whether or not they already have a role. Note that this is _not_ a list
     * of players to be assigned a role from this team, but it is instead every possible
     * player who _could_ be assigned one.
     * @param settings Role settings to use when calculating how many players
     * should assigned a specific role.
     * @param roleTeam The team of roles to assign.
     * @param maxAssignable The maximum number of players that can be assigned
     * a role from this team. For example, it could be the set number of impostors
     * configured in the game settings.
     * @param defaultRole The default role to assign to each player
     * @param roleAssignments A map of role assigments (i.e. a map of player to
     * roles) to act as a collective output to this method.
     * @returns The role assignments (i.e. a map of player to roles) that have
     * either collectively been assigned (if roleAssignments is passed)
     * or been assigned just as part of this method.
     */
    getRoleAssignmentsForTeam(playerPool, settings, roleTeam, maxAssignable, defaultRole, roleAssignments = new Map) {
        const teamRoles = this.matchRoles(settings, roleCtr => roleCtr.roleMetadata.roleTeam === roleTeam && !roleCtr.roleMetadata.isGhostRole);
        return this.getRoleAssignmentsFromRoleList(playerPool, settings, teamRoles, maxAssignable, defaultRole, roleAssignments);
    }
    /**
     * Specifically assign a pool of players to each role in a list, with regard
     * to the room settings (i.e. the chance of each role appearing or the maximum
     * number of assignments for the role).
     * @param playerPool The entire list of players that can be assigned a role,
     * whether or not they already have a role. Note that this is _not_ a list
     * of players to be assigned a role from this team, but it is instead every possible
     * player who _could_ be assigned one.
     * @param settings Role settings to use when calculating how many players
     * should assigned a specific role.
     * @param roleTeam The team of roles to assign.
     * @param maxAssignable The maximum number of players that can be assigned
     * a role from this team. For example, it could be the set number of impostors
     * configured in the game settings.
     * @param defaultRole The default role to assign to each player
     * @param roleAssignments A map of role assigments (i.e. a map of player to
     * roles) to act as a collective output to this method.
     * @returns The role assignments (i.e. a map of player to roles) that have
     * either collectively been assigned (if roleAssignments is passed)
     * or been assigned just as part of this method.
     */
    getRoleAssignmentsFromRoleList(playerPool, settings, teamRoles, maxAssignable, defaultRole, roleAssignments = new Map) {
        const roleAssignmentData = [];
        for (let i = 0; i < teamRoles.length; i++) {
            const roleCtr = teamRoles[i];
            roleAssignmentData.push({
                roleCtr,
                chance: settings[roleCtr.roleMetadata.roleType].chance,
                count: settings[roleCtr.roleMetadata.roleType].maxPlayers
            });
        }
        const fullChanceRoles = [];
        const lesserChanceRoles = [];
        for (const roleAssignment of roleAssignmentData) {
            if (roleAssignment.chance >= 100) {
                for (let i = 0; i < roleAssignment.count; i++) {
                    fullChanceRoles.push(roleAssignment.roleCtr);
                    roleAssignment.count--;
                }
            }
            else if (roleAssignment.chance > 0) {
                for (let i = 0; i < roleAssignment.count; i++) {
                    if (Math.random() * 101 < roleAssignment.chance) {
                        lesserChanceRoles.push(roleAssignment.roleCtr);
                    }
                }
            }
        }
        this.getRoleAssignmentsForPlayers(playerPool, maxAssignable, fullChanceRoles, roleAssignments);
        this.getRoleAssignmentsForPlayers(playerPool, maxAssignable, lesserChanceRoles, roleAssignments);
        if (defaultRole) {
            const defaultRoles = [];
            const num = Math.min(playerPool.length, maxAssignable);
            for (let i = 0; i < num; i++) {
                defaultRoles.push(defaultRole);
            }
            this.getRoleAssignmentsForPlayers(playerPool, maxAssignable, defaultRoles, roleAssignments);
        }
        return roleAssignments;
    }
    /**
     * Assign a list of roles to a pool of players, eliminating both players
     * from the player list and also roles from the role list, and without regard
     * for room settings (i.e. the chances of each role appearing or the maximum
     * number of assignments for the role).
     * @param playerPool The pool of players to assign a list of roles to.
     * @param maxAssignable The maximum number of players that cn be assigned a
     * role from this list.
     * @param roleList The list of roles to assign to players.
     * @param roleAssignments A map of role assigments (i.e. a map of player to
     * roles) to act as a collective output to this method.
     * @returns The role assignments (i.e. a map of player to roles) that have
     * either collectively been assigned (if roleAssignments is passed)
     * or been assigned just as part of this method.
     */
    getRoleAssignmentsForPlayers(playerPool, maxAssignable, roleList, roleAssignments = new Map) {
        let numAssigned = 0;
        while (roleList.length > 0 && playerPool.length > 0 && numAssigned < maxAssignable) {
            const roleIdx = Math.floor(Math.random() * roleList.length);
            const roleCtr = roleList[roleIdx];
            roleList.splice(roleIdx, 1);
            const playerIdx = Math.floor(Math.random() * playerPool.length);
            const player = playerPool[playerIdx];
            playerPool.splice(playerIdx, 1);
            roleAssignments.set(player, roleCtr);
            numAssigned++;
        }
        return roleAssignments;
    }
    /**
     * Randomly assign players to each enabled role with a certain probability.
     * Called just after a game is started and emits a {@link RoomAssignRolesEvent}
     * which can be used to alter which players are assigned which roles.
     */
    async assignRoles() {
        var _a, _b;
        const allPlayers = [];
        for (const [, player] of this.room.players) {
            if (!((_a = player.playerInfo) === null || _a === void 0 ? void 0 : _a.isDisconnected) && !((_b = player.playerInfo) === null || _b === void 0 ? void 0 : _b.isDead)) {
                allPlayers.push(player);
            }
        }
        const roleAssignments = new Map;
        const adjustedImpostors = allPlayers.length < 7 ? 1 : allPlayers.length < 9 ? 2 : 3;
        const assignedImpostors = this.getRoleAssignmentsForTeam(allPlayers, this.room.settings.roleSettings.roleChances, constant_1.RoleTeamType.Impostor, Math.min(adjustedImpostors, this.room.settings.numImpostors), roles_1.ImpostorRole);
        const assignedCrewmates = this.getRoleAssignmentsForTeam(allPlayers, this.room.settings.roleSettings.roleChances, constant_1.RoleTeamType.Crewmate, 2 ** 31 - 1, roles_1.CrewmateRole);
        for (const [player, roleCtr] of assignedImpostors) {
            roleAssignments.set(player, roleCtr);
        }
        for (const [player, roleCtr] of assignedCrewmates) {
            roleAssignments.set(player, roleCtr);
        }
        const ev = await this.emit(new events_1.RoomAssignRolesEvent(this.room, roleAssignments));
        if (!ev.canceled) {
            await this.assignRolesFromAssignments(ev.alteredAssignments);
        }
    }
    /**
     * Try to assign a ghost role to a specific dead player.
     * @param player The player to assign the role to.
     */
    async tryAssignGhostRole(player) {
        var _a, _b;
        if (!((_a = player.playerInfo) === null || _a === void 0 ? void 0 : _a.isDead) || player.playerInfo.isImpostor)
            return;
        const ghostRoles = this.matchRoles(this.room.settings.roleSettings.roleChances, (roleCtr, roleChance) => {
            if (!roleCtr.roleMetadata.isGhostRole)
                return false;
            if (roleChance.chance >= 100)
                return true;
            if (Math.random() * 101 < roleChance.chance)
                return true;
            return false;
        });
        if (!ghostRoles.length)
            return;
        const randomRole = ghostRoles[Math.floor(Math.random() * ghostRoles.length)];
        await ((_b = player.control) === null || _b === void 0 ? void 0 : _b.setRole(randomRole));
    }
    /**
     * Actually set all player roles from a map of player to role assignments,
     * can be gathered from {@link InnerShipStatus.getRoleAssignmentsForTeam}.
     * @param roleAssignments A map of player to role assignments to assign to
     * every role.
     */
    async assignRolesFromAssignments(roleAssignments) {
        var _a;
        const promises = [];
        for (const [player, roleCtr] of roleAssignments) {
            promises.push((_a = player.control) === null || _a === void 0 ? void 0 : _a.setRole(roleCtr));
        }
        await Promise.all(promises);
    }
    addTasksFromList(start, count, tasks, usedTaskTypes, unusedTasks) {
        if (unusedTasks.length === 0) {
            return start;
        }
        let numLoops = 0; // fun fact: nodepolus calls this a "sanity check"
        let i = 0;
        while (i < count && numLoops++ < 1000) {
            if (start >= unusedTasks.length) {
                start = 0;
                shuffleArray(unusedTasks);
                if (unusedTasks.every(taskData => usedTaskTypes.has(taskData.taskType))) {
                    usedTaskTypes.clear();
                }
            }
            const task = unusedTasks[start++];
            if (usedTaskTypes.has(task.taskType)) {
                i--;
            }
            else {
                usedTaskTypes.add(task.taskType);
                tasks.push(task.index);
            }
            i++;
        }
        return start;
    }
    /**
     * Randomly assign tasks to all players, using data from @skeldjs/data.
     */
    assignTasks() {
        const allTasks = this.getTasks();
        const numCommon = this.room.settings.commonTasks;
        const numLong = this.room.settings.longTasks;
        const numShort = this.room.settings.shortTasks;
        const allCommon = [];
        const allLong = [];
        const allShort = [];
        for (let i = 0; i < allTasks.length; i++) {
            switch (allTasks[i].length) {
                case constant_1.TaskLength.Common:
                    allCommon.push(allTasks[i]);
                    break;
                case constant_1.TaskLength.Long:
                    allLong.push(allTasks[i]);
                    break;
                case constant_1.TaskLength.Short:
                    allShort.push(allTasks[i]);
                    break;
            }
        }
        shuffleArray(allCommon);
        shuffleArray(allLong);
        shuffleArray(allShort);
        const usedTaskTypes = new Set;
        const commonTasks = [];
        this.addTasksFromList(0, numCommon, commonTasks, usedTaskTypes, allCommon);
        let shortIdx = 0;
        let longIdx = 0;
        for (const [, player] of this.room.players) {
            if (!player.playerInfo)
                continue;
            usedTaskTypes.clear();
            const playerTasks = [...commonTasks];
            shortIdx = this.addTasksFromList(shortIdx, numShort, playerTasks, usedTaskTypes, allShort);
            longIdx = this.addTasksFromList(longIdx, numLong, playerTasks, usedTaskTypes, allLong);
            player.playerInfo.setTaskIds(playerTasks);
        }
    }
    /**
     * Get the spawn position of a player whether they are about to spawn after
     * starting or whether they are about to spawn after a meeting.
     * @param player The player or player ID to determine the position of.
     * @param initialSpawn Whther or not this is a spawn after starting the game.
     * @returns The spawn position of the player.
     */
    getSpawnPosition(player, initialSpawn) {
        const playerId = typeof player === "number"
            ? player
            : player.playerId;
        return util_1.Vector2.up
            .rotateDeg((playerId - 1) * (360 / this.room.players.size))
            .mul(this.spawnRadius)
            .add(initialSpawn
            ? this.initialSpawnCenter
            : this.meetingSpawnCenter)
            .add(new util_1.Vector2(0, 0.3636));
    }
    /**
     * Teleport a player to their spawn position, calculated using
     * {@link InnerShipStatus.getSpawnPosition}.
     * @param player The player to determine the position of.
     * @param initialSpawn Whether or not this is a spawn after starting the game.
     */
    spawnPlayer(player, initialSpawn) {
        var _a;
        if (player.playerId === undefined)
            return;
        (_a = player.transform) === null || _a === void 0 ? void 0 : _a.snapTo(this.getSpawnPosition(player, initialSpawn), false);
    }
}
exports.InnerShipStatus = InnerShipStatus;
//# sourceMappingURL=InnerShipStatus.js.map