"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.LifeSuppSystem = void 0;
const constant_1 = require("@skeldjs/constant");
const SystemStatus_1 = require("./SystemStatus");
const events_1 = require("../events");
const endgame_1 = require("../endgame");
/**
 * Represents a system responsible for handling oxygen consoles.
 *
 * See {@link LifeSuppSystemEvents} for events to listen to.
 */
class LifeSuppSystem extends SystemStatus_1.SystemStatus {
    constructor(ship, systemType, data) {
        var _a;
        super(ship, systemType, data);
        this.lastUpdate = 0;
        (_a = this.timer) !== null && _a !== void 0 ? _a : (this.timer = 10000);
        this.completed || (this.completed = new Set);
    }
    get sabotaged() {
        return this.timer < 10000;
    }
    /* eslint-disable-next-line @typescript-eslint/no-unused-vars */
    Deserialize(reader, spawn) {
        const timer = this.timer;
        this.timer = reader.float();
        const num_consoles = reader.upacked();
        if (this.completed.size > 0 && num_consoles === 0) {
            this._clearConsoles(undefined, undefined);
        }
        else {
            for (let i = 0; i < num_consoles; i++) {
                const consoleId = reader.upacked();
                this._completeConsole(consoleId, undefined, undefined);
            }
        }
        if (timer === 10000 && this.timer < 10000) {
            this.emitSync(new events_1.SystemSabotageEvent(this.room, this, undefined, undefined));
        }
        else if (timer < 10000 && this.timer === 10000) {
            this.emitSync(new events_1.SystemRepairEvent(this.room, this, undefined, undefined));
        }
    }
    /* eslint-disable-next-line @typescript-eslint/no-unused-vars */
    Serialize(writer, spawn) {
        writer.float(this.timer);
        writer.upacked(this.completed.size);
        for (const console of this.completed) {
            writer.upacked(console);
        }
    }
    async HandleSabotage(player, rpc) {
        this.timer = 45;
        const oldCompleted = this.completed;
        this._clearConsoles(player, rpc);
        const ev = await this.emit(new events_1.SystemSabotageEvent(this.room, this, rpc, player));
        if (ev.reverted) {
            this.timer = 10000;
            this.completed = oldCompleted;
        }
    }
    async _clearConsoles(player, rpc) {
        const completedBefore = new Set(this.completed);
        this.completed = new Set;
        this.dirty = true;
        const ev = await this.emit(new events_1.O2ConsolesClearEvent(this.room, this, rpc, player));
        if (ev.reverted) {
            this.completed = completedBefore;
        }
    }
    /**
     * Clear the completed consoles. This is a host operation on official servers.
     */
    async clearConsoles() {
        await this._clearConsoles(this.room.myPlayer, undefined);
    }
    async _completeConsole(consoleId, player, rpc) {
        this.completed.add(consoleId);
        this.dirty = true;
        const ev = await this.emit(new events_1.O2ConsolesCompleteEvent(this.room, this, undefined, player, consoleId));
        if (ev.reverted) {
            this.completed.delete(ev.consoleId);
            return;
        }
        if (ev.alteredConsoleId !== consoleId) {
            this.completed.delete(consoleId);
            this.completed.add(ev.alteredConsoleId);
        }
        if (this.completed.size >= 2) {
            await this._repair(player, rpc);
        }
    }
    /**
     * Mark a console as being complete.
     * @param consoleId The ID of the console to mark as complete.
     */
    async completeConsole(consoleId) {
        if (this.room.hostIsMe) {
            await this._completeConsole(consoleId, this.room.myPlayer, undefined);
        }
        else {
            await this._sendRepair(0x40 | consoleId);
        }
    }
    async _repair(player, rpc) {
        const oldTimer = this.timer;
        const oldCompleted = this.completed;
        this.timer = 10000;
        this.completed = new Set;
        this.dirty = true;
        const ev = await this.emit(new events_1.O2ConsolesClearEvent(this.room, this, rpc, player));
        if (ev.reverted) {
            this.timer = oldTimer;
            this.completed = oldCompleted;
        }
    }
    async repair() {
        if (this.room.hostIsMe) {
            this._repair(this.room.myPlayer, undefined);
        }
        else {
            await this._sendRepair(0x10);
        }
    }
    async HandleRepair(player, amount, rpc) {
        const consoleId = amount & 0x3;
        if (amount & 0x40) {
            await this._completeConsole(consoleId, player, rpc);
        }
        else if (amount & 0x10) {
            await this._repair(player, rpc);
        }
    }
    Detoriorate(delta) {
        if (!this.sabotaged)
            return;
        this.timer -= delta;
        this.lastUpdate += delta;
        if (this.lastUpdate > 2) {
            this.lastUpdate = 0;
            this.dirty = true;
        }
        if (this.timer <= 0) {
            this.room.registerEndGameIntent(new endgame_1.EndGameIntent(endgame_1.AmongUsEndGames.O2Sabotage, constant_1.GameOverReason.ImpostorBySabotage, {}));
        }
    }
}
exports.LifeSuppSystem = LifeSuppSystem;
//# sourceMappingURL=LifeSuppSystem.js.map